from autodocgenerator.manage import Manager
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn, TimeRemainingColumn
from autodocgenerator.factory.base_factory import DocFactory
from autodocgenerator.factory.modules.intro import IntroLinks, IntroText
from autodocgenerator.ui.progress_base import BaseProgress, LibProgress, ConsoleGtiHubProgress
from autodocgenerator.preprocessor.settings import ProjectSettings
from .config_reader import Config, read_config, ProjectConfigSettings
from autodocgenerator.engine.models.gpt_model import GPTModel, AsyncGPTModel
from autodocgenerator.engine.config.config import API_KEY
from autodocgenerator.ui.logging import BaseLogger, InfoLog, ErrorLog, WarningLog, FileLoggerTemplate
import os


def gen_doc(project_settings: ProjectSettings, pcs: ProjectConfigSettings, ignore_list: list[str], project_path: str, doc_factory: DocFactory, intro_factory: DocFactory):
    
    sync_model = GPTModel(API_KEY, use_random=False)
    async_model = AsyncGPTModel(API_KEY)
    
    manager = Manager(
        project_path, 
        project_settings,
        pcs, 
        sync_model=sync_model,
        async_model=async_model,
        ignore_files=ignore_list, 
        progress_bar=ConsoleGtiHubProgress(), 
        language="en")


    manager.generate_code_file()
    manager.generate_global_info_file(use_async=False, max_symbols=8000)
    manager.generete_doc_parts(use_async=False, max_symbols=6000)
    manager.factory_generate_doc(doc_factory)
    manager.order_doc()
    manager.factory_generate_doc(intro_factory)
    manager.clear_cache()

    return manager.read_file_by_file_key("output_doc")

if __name__ == "__main__":
    with open("autodocconfig.yml", "r", encoding="utf-8") as file:
        config_data = file.read()
    config: Config = read_config(config_data)


    project_settings = config.get_project_settings()
    doc_factory, intro_factory = config.get_doc_factory()

    output_doc = gen_doc(
        project_settings,
        config.pcs,
        config.ignore_files,
        ".",
        doc_factory,
        intro_factory
    )

    






from .model import Model, AsyncModel, API_KEY, History
from groq import Groq, AsyncGroq
from ..exceptions import ModelExhaustedException
from ...ui.logging import BaseLogger, InfoLog, ErrorLog, WarningLog

class AsyncGPTModel(AsyncModel):
    def __init__(self, api_key=API_KEY, history = History(), use_random: bool = True):
        super().__init__(api_key, history, use_random)
        self.client = AsyncGroq(api_key=self.api_key)
        self.logger = BaseLogger()

    async def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        
        self.logger.log(InfoLog("Generating answer asynchronously..."))

        if with_history:
            messages = self.history.history
        else:
            messages = prompt
        
        chat_completion = None
        model_name = None
        while True:
            if len(self.regen_models_name) <= 0:
                self.logger.log(ErrorLog("No models available for use."))
                raise ModelExhaustedException("No models available for use.")
            
            model_name = self.regen_models_name[self.current_model_index]
            try:
                chat_completion = await self.client.chat.completions.create(
                    messages=messages,
                    model=model_name,
                )
                break
            except Exception as e:
                print(e)
                self.logger.log(WarningLog(f"Model {model_name} failed with error: {str(e)}. Trying next model..."))
                self.current_model_index = 0 if self.current_model_index + 1 >= len(self.regen_models_name) else self.current_model_index + 1
                    


        result = chat_completion.choices[0].message.content
        self.logger.log(InfoLog(f"Generated answer with model {model_name}."))
        self.logger.log(InfoLog(f"Answer: {result}", level=2))

        return result

class GPTModel(Model):
    def __init__(self, api_key=API_KEY, history = History(), use_random: bool = True):
        super().__init__(api_key, history, use_random)
        self.client = Groq(api_key=self.api_key)
        self.logger = BaseLogger()


    def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        self.logger.log(InfoLog("Generating answer..."))
        if with_history:
            messages = self.history.history
        else:
            messages = prompt
        
        chat_completion = None
        model_name = None

        while True:
            if len(self.regen_models_name) <= 0:
                self.logger.log(ErrorLog("No models available for use."))
                raise ModelExhaustedException("No models available for use.")
            
            model_name = self.regen_models_name[self.current_model_index]
            try:
                chat_completion = self.client.chat.completions.create(
                    messages=messages,
                    model=model_name,

                )
                break
            except Exception as e:
                print(e)
                self.logger.log(WarningLog(f"Model {model_name} failed with error: {str(e)}. Trying next model..."))
                self.current_model_index = 0 if self.current_model_index + 1 >= len(self.regen_models_name) else self.current_model_index + 1


        result = chat_completion.choices[0].message.content
        self.logger.log(InfoLog(f"Generated answer with model {model_name}."))
        self.logger.log(InfoLog(f"Answer: {result}", level=2))
        return result


from rich.progress import Progress


class BaseProgress:
    def __init__(self):
        pass

    def create_new_subtask(self, name: str, total_len: int):
        ...

    def update_task(self):
        ...

    def remove_subtask(self):
        ...

class LibProgress(BaseProgress):
    
    def __init__(self, progress: Progress, total=4):
        super().__init__()
        self.progress = progress
        self._base_task = self.progress.add_task("General progress", total=total)
        self._cur_sub_task = None

    def create_new_subtask(self, name, total_len):
        self._cur_sub_task = self.progress.add_task(name, total=total_len)

    def update_task(self):
        if self._cur_sub_task is None:
            self.progress.update(self._base_task, advance=1)
        else:
            self.progress.update(self._cur_sub_task, advance=1)

    def remove_subtask(self):
        if self._cur_sub_task is not None:
            self._cur_sub_task = None


class ConsoleTask:
    def __init__(self, name, total_len):
        self.name = name
        self.total_len = total_len
        self.start_task()

    def start_task(self):
        self.current_len = 0
        print(f"Starting task: {self.name} with total length {self.total_len}")

    def progress(self):
        self.current_len += 1
        percent = (self.current_len / self.total_len) * 100
        print(f"Task: {self.name} Progress: {percent:.1f}%")

    

class ConsoleGtiHubProgress(BaseProgress):
    def __init__(self):
        super().__init__()
        self.curr_task = None
        self.gen_task = ConsoleTask("General Progress", 5)

    def create_new_subtask(self, name: str, total_len: int):
        self.curr_task = ConsoleTask(name, total_len)

    def update_task(self):
        if self.curr_task is not None:
            self.curr_task.progress()
        else:
            self.gen_task.progress()


    def remove_subtask(self):
        self.curr_task = None